from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _
from multiselectfield import MultiSelectField
from .enums import NetworksNames


class UserProxy(User):
    class Meta:
        proxy = True
        verbose_name = _("Operator")
        verbose_name_plural = _("Operators")


class OrganizationKeys(models.Model):
    address = models.CharField(max_length=100)
    private_key = models.CharField(max_length=500)
    public_key = models.CharField(max_length=500)
    mnemonic = models.TextField()

    def __str__(self) -> str:
        return self.address


class Organization(models.Model):
    networks = MultiSelectField(
        choices=NetworksNames.choices(), max_choices=2, max_length=1000
    )
    name = models.CharField(max_length=255)
    email = models.EmailField()
    phone = models.CharField(max_length=255)
    keys = models.ForeignKey(
        OrganizationKeys, on_delete=models.SET_NULL, blank=True, null=True
    )
    operators = models.ManyToManyField(UserProxy)

    def __str__(self) -> str:
        return self.name


class OrganizationDID(models.Model):
    network_name = models.CharField(max_length=2500)
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE)
    did = models.CharField(max_length=255)

    def __str__(self) -> str:
        return self.organization.name


class Issuer(models.Model):
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE)
    active = models.BooleanField(default=True)

    def __str__(self) -> str:
        return self.organization.name


class Intermediary(models.Model):
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE)
    active = models.BooleanField(default=True)

    def __str__(self) -> str:
        return self.organization.name



from django.contrib import admin
from .models import (
    Intermediary,
    Organization,
    OrganizationDID,
    OrganizationKeys,
    Issuer,
    UserProxy,
)
from .enums import OrganizationsSwitches
import waffle


class IntermediaryAdmin(admin.ModelAdmin):
    model = Intermediary

    def get_model_perms(self, request):

        if (
            not waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            or waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            and not waffle.switch_is_active(
                OrganizationsSwitches.OrganizationsIntermediary.value
            )
        ):
            return {}

        return super(IntermediaryAdmin, self).get_model_perms(request)


class OrganizationAdmin(admin.ModelAdmin):
    model = Organization

    def get_model_perms(self, request):

        if not waffle.switch_is_active(OrganizationsSwitches.Organizations.value):
            return {}

        return super(OrganizationAdmin, self).get_model_perms(request)


class OrganizationDIDAdmin(admin.ModelAdmin):
    model = OrganizationDID

    def get_model_perms(self, request):

        if not waffle.switch_is_active(OrganizationsSwitches.Organizations.value):
            return {}

        return super(OrganizationDIDAdmin, self).get_model_perms(request)


class OrganizationKeysAdmin(admin.ModelAdmin):
    model = OrganizationKeys

    def get_model_perms(self, request):

        if not waffle.switch_is_active(OrganizationsSwitches.Organizations.value):
            return {}

        return super(OrganizationKeysAdmin, self).get_model_perms(request)


class IssuerAdmin(admin.ModelAdmin):
    model = Issuer

    def get_model_perms(self, request):

        if (
            not waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            or waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            and not waffle.switch_is_active(
                OrganizationsSwitches.OrganizationsIssuer.value
            )
        ):
            return {}

        return super(IssuerAdmin, self).get_model_perms(request)


class UserProxyAdmin(admin.ModelAdmin):
    model = UserProxy
    fields = ("username", "first_name", "last_name", "email")

    def get_model_perms(self, request):

        if not waffle.switch_is_active(OrganizationsSwitches.Organizations.value):
            return {}

        return super(UserProxyAdmin, self).get_model_perms(request)


admin.site.register(Intermediary, IntermediaryAdmin)
admin.site.register(Organization, OrganizationAdmin)
admin.site.register(OrganizationDID, OrganizationDIDAdmin)
admin.site.register(OrganizationKeys, OrganizationKeysAdmin)
admin.site.register(Issuer, IssuerAdmin)
admin.site.register(UserProxy, UserProxyAdmin)



from abc import ABC, abstractmethod
from .models import FactoryArgsModel
from alastria_service_client.validators import Address


class ANetworkStrategy(ABC):
    @abstractmethod
    def create_did(
        self,
        sign_address: Address = Address(""),
        issuer_address: Address = Address(""),
        public_key: str = "",
        issuer_private_key: str = "",
        new_issuer_private_key: str = "",
    ) -> str:
        """"""


class AContext(ABC):
    @abstractmethod
    def request(self) -> ANetworkStrategy:
        """"""


class AFactory(ABC):
    @staticmethod
    @abstractmethod
    def create_object(props: FactoryArgsModel) -> AContext:
        """"""



from celery import shared_task
from django.conf import settings
from network_service_client.client import (
    Client as NetworkClient,
    Network as NetworkDTO,
    NetworksNames,
)
from organizations.did_factory.models import FactoryArgsModel
from organizations.did_factory.factory import Creator
from organizations.models import Organization, OrganizationDID


@shared_task
def create_organization_did(organization_id: int) -> None:
    organization = Organization.objects.get(pk=organization_id)
    network_data: NetworkDTO = NetworkClient(
        service_host=settings.NETWORK_SERVICE_HOST
    ).get_network_by_name(
        NetworksNames.AlastriaDefaultName
    )  # TODO ADD LACC SOME DAY ?
    props = FactoryArgsModel(net=network_data)
    context = Creator().create_object(props).request()
    did: str = context.create_did(
        organization.keys.address,
        settings.ISSUER_ADDRESS,
        str(organization.keys.public_key),
        settings.ISSUER_PRIVATE_KEY,
        organization.keys.private_key,  # TODO: encrypt this
    )

    organization_did = OrganizationDID(
        network_name=NetworksNames.AlastriaDefaultName,
        organization=organization,
        did=did,
    )

    organization_did.save()



from enum import Enum


class OrganizationsSwitches(str, Enum):
    Organizations = "organizations manager"
    OrganizationsIssuer = "organizations issuer manager"
    OrganizationsIntermediary = "organizations intermediary manager"


class NetworksNames(str, Enum):
    AlastriaDefaultName = "Alastria"
    LacchainDefaultName = "Lacchain"

    @classmethod
    def choices(cls):
        return tuple((i.name, i.value) for i in cls)



# Generated by Django 4.1.3 on 2023-01-20 10:26

from django.db import migrations
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ("organizations", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="networks",
            field=multiselectfield.db.fields.MultiSelectField(
                choices=[
                    ("AlastriaDefaultName", "Alastria"),
                    ("LacchainDefaultName", "Lacchain"),
                ],
                default=1,
                max_length=19,
            ),
            preserve_default=False,
        ),
    ]
